/*
 * Copyright (C) 2016 Centro de Computacao Cientifica e Software Livre
 * Departamento de Informatica - Universidade Federal do Parana
 *
 * This file is part of blend.
 *
 * blend is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * blend is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with blend.  If not, see <http://www.gnu.org/licenses/>.
 */

import { Dimension } from "./dimension";
import { Metric } from "./metric";
import { View } from "./view";
import { Query } from "../common/query";

export class Engine {
    private views: View[] = [];
    private metrics: Metric[] = [];
    private dimensions: Dimension[] = [];

    constructor () { }

    public getViews() {
        return this.views;
    }

    public addView(view: View) {
        this.views.push(view);
        return view;
    }

    public addMetric(metric: Metric) {
        this.metrics.push(metric);
        return metric;
    }

    public getMetricByName(name: string) {
        let result = this.metrics.find(metric => metric.name === name);

        if (!result) {
            throw new Error("The metric named " + name + " was not found");
        }

        return result;
    }

    public addDimension(dimension: Dimension) {
        this.dimensions.push(dimension);
        return dimension;
    }

    public getDimensionByName(name: string) {
        let result = this.dimensions.find(dimension => dimension.name === name);

        if (!result) {
            throw new Error("The dimension named " + name + " was not found");
        }

        return result;
    }

    public query (q: Query) {
        return this.selectOptimalView(q);
    }

    private selectOptimalView (q: Query) {
        let metricsName = q.metrics.map((met) => met.name);
        let dimensionsName = q.dimensions.map((dim) => dim.name);
        let objective = metricsName.concat(dimensionsName);
        let optimalViews: View[] = [];
        let activeViews = this.getViews();

        // run this block until all metrics and dimmensions are covered
        while (objective.length  > 0) {
            let bestView: View;
            let shortestDistance = objective.length + 1;

            // remove views from the activeViews if they don't intersect
            // with the objective
            activeViews = activeViews.filter((view: View) => {
                metricsName = view.metrics.map((met) => met.name);
                dimensionsName = view.dimensions.map((dim) => dim.name);
                let cover = metricsName.concat(dimensionsName);
                let intersection = cover.filter((item: string) => {
                    return objective.indexOf(item) !== -1;
                });

                if (intersection.length > 0) {
                    let distance = objective.length - intersection.length;

                    if (distance < shortestDistance) {
                        bestView = view;
                        shortestDistance = distance;
                    }
                    return true;
                }

                /*If the intersection is empty,
                  remove this element from future searches*/
                return false;
            });

            if (shortestDistance  === objective.length + 1) {
                throw new Error("Engine views cannot cover the query");
            }

            optimalViews.push(bestView);

            // remove metrics and dimensions corvered by the bestView from the
            // objective (i.e. the object is already met for those metrics/dimensions)
            objective = objective.filter((item: string) => {
                metricsName = bestView.metrics.map((met) => met.name);
                dimensionsName = bestView.dimensions.map((dim) => dim.name);
                let cover = dimensionsName.concat(metricsName);
                return cover.indexOf(item) === -1;
            });
        }

        if (optimalViews.length === 1) {
            // if there is a single view that covers the query, we just return it
            return optimalViews.pop();
        }
        else {
            // if more than one view is necessary to cover the query,
            // we need to compose them into a new singular virtual view

            let options = {
                metrics: q.metrics,
                dimensions: q.dimensions,
                materialized: false,
                childViews: optimalViews
            };

            let view = new View(options);
            this.addView(view);
            return view;
        }
    }
}
